""" in-wheel-motored vehicle """


class InWheelMotoredVehicle:
    """ class to define in-wheel-motored vehicle model """
    def __init__(self) -> None:
        # under developing
        raise NotImplementedError
